"""medicationrequests serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer
from rest_framework import serializers

from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    OrganizationReferenceSerializer,
    PeriodSerializer,
    QuantitySerializer,
    ReferenceSerializer,
)
from dfhir.base.serializers import QuantitySerializer as DurationSerializer
from dfhir.devices.serializers import DeviceCodeableReferenceSerializer
from dfhir.encounters.serializers import EncounterReferenceSerializer
from dfhir.medicationrequests.models import (
    AdditionalIllustration,
    DispenseRequest,
    DispenseRequestInitialFill,
    DosageMethod,
    DosageRoute,
    DosageSite,
    MedicationRequest,
    MedicationRequestBasedOnReference,
    MedicationRequestCategory,
    MedicationRequestInsuranceReference,
    MedicationRequestMedicationCode,
    MedicationRequestPerformerReference,
    MedicationRequestProvinanceReference,
    MedicationRequestReason,
    MedicationRequestReasonReference,
    MedicationRequestReference,
    MedicationRequestReferenceType,
    MedicationRequestRequesterReference,
    MedicationRequestSubstitution,
    MedicatonRequestReasonCodealbleReference,
    ReferenceAsNeededFor,
    medicationRequestInformationSourceReference,
)
from dfhir.medications.serializers import MedicationCodeableReferenceSerializer
from dfhir.patients.serializers import (
    PatientGroupReferenceSerializer as SubjectSerializer,
)
from dfhir.practitioners.serializers import (
    PractitionerPractitionerRoleReferenceSerializer,
)


class MedicationRequestBasedOnReferenceSerializer(BaseReferenceModelSerializer):
    """medication request based on reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """metadata."""

        model = MedicationRequestBasedOnReference
        exclude = ["created_at", "updated_at"]


class MedicationRequestReferenceSerializer(BaseReferenceModelSerializer):
    """medication request reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """metadata."""

        model = MedicationRequestReference
        exclude = ["created_at", "updated_at"]


class MedicationRequestInformationSourceReferenceSerializer(
    BaseReferenceModelSerializer
):
    """medication request information source reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """metadata."""

        model = medicationRequestInformationSourceReference
        exclude = ["created_at", "updated_at"]


class MedicationRequestRequesterReferenceSerializer(BaseReferenceModelSerializer):
    """medication request requester reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """metadata."""

        model = MedicationRequestRequesterReference
        exclude = ["created_at", "updated_at"]


class MedicationRequestPerformerReferenceSerializer(BaseReferenceModelSerializer):
    """medication request performer reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """metadata."""

        model = MedicationRequestPerformerReference
        exclude = ["created_at", "updated_at"]


class MedicationRequestReasonReferenceSerializer(BaseReferenceModelSerializer):
    """medication request reason reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """metadata."""

        model = MedicationRequestReasonReference
        exclude = ["created_at", "updated_at"]


class MedicatonRequestReasonCodealbleReferenceSerializer(WritableNestedModelSerializer):
    """medication request reason codeable reference serializer."""

    concept = CodeableConceptSerializer(many=False, required=False)
    reference = MedicationRequestReasonReferenceSerializer(many=False, required=False)

    class Meta:
        """metadata."""

        model = MedicatonRequestReasonCodealbleReference
        exclude = ["created_at", "updated_at"]


class DispenseRequestInitialFillSerializer(serializers.ModelSerializer):
    """dispense request initial fill serializer."""

    quantity = QuantitySerializer(many=False, required=False)
    duration = DurationSerializer(many=False, required=False)

    class Meta:
        """metadata."""

        model = DispenseRequestInitialFill
        exclude = ["created_at", "updated_at"]


class DispenseRequestSerializer(WritableNestedModelSerializer):
    """dispense request serializer."""

    initial_fill = DispenseRequestInitialFillSerializer(many=False, required=False)
    dispense_interval = DurationSerializer(many=False, required=False)
    validity_period = PeriodSerializer(many=False, required=False)
    quantity = QuantitySerializer(many=False, required=False)
    expected_supply_duration = DurationSerializer(many=False, required=False)
    dispenser = OrganizationReferenceSerializer(many=False, required=False)
    dispense_instruction = AnnotationSerializer(many=False, required=False)
    dose_administration = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """metadata."""

        model = DispenseRequest
        exclude = ["created_at", "updated_at"]


class MedicationRequestSubstitutionSerializer(WritableNestedModelSerializer):
    """medication request substitution serializer."""

    allowed_codeable_concept = CodeableConceptSerializer(many=False, required=False)
    reason = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """metadata."""

        model = MedicationRequestSubstitution
        exclude = ["created_at", "updated_at"]


class MedicationRequestInsuranceReferenceSerializer(BaseReferenceModelSerializer):
    """medication request insurance reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """metadata."""

        model = MedicationRequestInsuranceReference
        exclude = ["created_at", "updated_at"]


class MedicationRequestProvinanceReferenceSerializer(BaseReferenceModelSerializer):
    """medication request provenance reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """metadata."""

        model = MedicationRequestProvinanceReference
        exclude = ["created_at", "updated_at"]


class MedicationRequestSerializer(BaseWritableNestedModelSerializer):
    """medication requests serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    based_on = MedicationRequestBasedOnReferenceSerializer(many=True, required=False)
    prior_prescription = MedicationRequestReferenceSerializer(
        many=False, required=False
    )
    group_identifier = IdentifierSerializer(many=False, required=False)
    status_reason = CodeableConceptSerializer(many=False, required=False)
    category = CodeableConceptSerializer(many=True, required=False)
    medication = MedicationCodeableReferenceSerializer(many=False, required=False)
    subject = SubjectSerializer(many=False, required=False)
    information_source = MedicationRequestInformationSourceReferenceSerializer(
        many=True, required=False
    )
    encounter = EncounterReferenceSerializer(many=False, required=False)
    supporting_information = ReferenceSerializer(many=True, required=False)
    requester = MedicationRequestRequesterReferenceSerializer(
        many=False, required=False
    )
    performer_type = CodeableConceptSerializer(many=False, required=False)
    performer = MedicationRequestPerformerReferenceSerializer(
        many=False, required=False
    )
    device = DeviceCodeableReferenceSerializer(many=True, required=False)
    recorder = PractitionerPractitionerRoleReferenceSerializer(
        many=False, required=False
    )
    reason = MedicatonRequestReasonCodealbleReferenceSerializer(
        many=True, required=False
    )
    course_of_therapy_type = CodeableConceptSerializer(many=False, required=False)
    insurance = MedicationRequestInsuranceReferenceSerializer(many=True, required=False)
    note = AnnotationSerializer(many=True, required=False)
    effective_dose_period = PeriodSerializer(many=False, required=False)
    dispense_request = DispenseRequestSerializer(many=False, required=False)
    substitution = MedicationRequestSubstitutionSerializer(many=False, required=False)
    event_history = MedicationRequestProvinanceReferenceSerializer(
        many=True, required=False
    )

    class Meta:
        """metadata."""

        model = MedicationRequest
        exclude = ["created_at", "updated_at"]


class MedicationRequestCategorySerializer(serializers.ModelSerializer):
    """medication request category serializer."""

    class Meta:
        """metadata."""

        model = MedicationRequestCategory
        exclude = ["created_at", "updated_at"]


class MedicationRequestMedicationCodeSerializer(serializers.ModelSerializer):
    """medication request medication code serializer."""

    class Meta:
        """metadata."""

        model = MedicationRequestMedicationCode
        exclude = ["created_at", "updated_at"]


class MediationRequestReferenceTypeSerializer(serializers.ModelSerializer):
    """medication request reference type serializer."""

    class Meta:
        """metadata."""

        model = MedicationRequestReferenceType
        exclude = ["created_at", "updated_at"]


class MedicationRequestReasonSerializer(serializers.ModelSerializer):
    """medication request reason serializer."""

    class Meta:
        """metadata."""

        model = MedicationRequestReason
        exclude = ["created_at", "updated_at"]


class AdditionalIllustrationSerializer(serializers.ModelSerializer):
    """additional illustration serializer."""

    class Meta:
        """metadata."""

        model = AdditionalIllustration
        exclude = ["created_at", "updated_at"]


class ReferenceAsNeededForSerializer(serializers.ModelSerializer):
    """reference as needed for serializer."""

    class Meta:
        """metadata."""

        model = ReferenceAsNeededFor
        exclude = ["created_at", "updated_at"]


class DosageSiteSerializer(serializers.ModelSerializer):
    """dosage site serializer."""

    class Meta:
        """metadata."""

        model = DosageSite
        exclude = ["created_at", "updated_at"]


class DosageRouteSerializer(serializers.ModelSerializer):
    """dosage route serializer."""

    class Meta:
        """metadata."""

        model = DosageRoute
        exclude = ["created_at", "updated_at"]


class DosageMethodSerializer(serializers.ModelSerializer):
    """dosage method serializer."""

    class Meta:
        """metadata."""

        model = DosageMethod
        exclude = ["created_at", "updated_at"]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('biologicallyderivedproducts', '0002_initial'),
        ('careplans', '0005_initial'),
        ('careteams', '0004_initial'),
        ('conditions', '0004_initial'),
        ('devices', '0001_initial'),
        ('diagnosticreports', '0003_initial'),
        ('groups', '0003_initial'),
        ('healthcareservices', '0001_initial'),
        ('locations', '0001_initial'),
        ('medicationadministrations', '0004_initial'),
        ('medicationrequests', '0003_initial'),
        ('medications', '0001_initial'),
        ('observations', '0003_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0002_initial'),
        ('practitionerroles', '0002_initial'),
        ('practitioners', '0001_initial'),
        ('procedures', '0001_initial'),
        ('relatedpersons', '0001_initial'),
        ('servicerequests', '0001_initial'),
        ('substances', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='procedure',
            name='follow_up',
            field=models.ManyToManyField(blank=True, related_name='procedure_follow_up', to='servicerequests.servicerequestplandefinitionreference'),
        ),
        migrations.AddField(
            model_name='procedure',
            name='identifier',
            field=models.ManyToManyField(blank=True, related_name='procedure_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='procedure',
            name='location',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_location', to='locations.locationreference'),
        ),
        migrations.AddField(
            model_name='procedure',
            name='note',
            field=models.ManyToManyField(blank=True, related_name='procedure_note', to='base.annotation'),
        ),
        migrations.AddField(
            model_name='procedure',
            name='occurrence_age',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_occurrence_age', to='base.age'),
        ),
        migrations.AddField(
            model_name='procedure',
            name='occurrence_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_occurrence_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='procedure',
            name='occurrence_range',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_occurrence_range', to='base.range'),
        ),
        migrations.AddField(
            model_name='procedure',
            name='occurrence_timing',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_occurrence_timing', to='base.timing'),
        ),
        migrations.AddField(
            model_name='procedure',
            name='outcome',
            field=models.ManyToManyField(blank=True, related_name='procedure_outcome', to='observations.observationcodeablereference'),
        ),
        migrations.AddField(
            model_name='procedure',
            name='status_reason',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_status_reason', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='procedure',
            name='supporting_info',
            field=models.ManyToManyField(blank=True, related_name='procedure_supporting_info', to='base.reference'),
        ),
        migrations.AddField(
            model_name='procedurebasedonreference',
            name='care_plan',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_based_on_reference_care_plan', to='careplans.careplan'),
        ),
        migrations.AddField(
            model_name='procedurebasedonreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_based_on_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='procedurebasedonreference',
            name='medication_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_based_on_reference_medication_request', to='medicationrequests.medicationrequest'),
        ),
        migrations.AddField(
            model_name='procedurebasedonreference',
            name='service_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_based_on_reference_service_request', to='servicerequests.servicerequest'),
        ),
        migrations.AddField(
            model_name='procedure',
            name='based_on',
            field=models.ManyToManyField(blank=True, related_name='procedure_based_on', to='procedures.procedurebasedonreference'),
        ),
        migrations.AddField(
            model_name='procedurecodeablereference',
            name='concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_codeable_reference_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='procedurefocaldevice',
            name='action',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_focal_device_action', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='procedurefocaldevice',
            name='manipulated',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_focal_device_manipulated', to='devices.devicereference'),
        ),
        migrations.AddField(
            model_name='procedure',
            name='focal_device',
            field=models.ManyToManyField(blank=True, related_name='procedure_focal_device', to='procedures.procedurefocaldevice'),
        ),
        migrations.AddField(
            model_name='procedurefocusreference',
            name='care_team',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_focus_reference_care_team', to='careteams.careteam'),
        ),
        migrations.AddField(
            model_name='procedurefocusreference',
            name='group',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_focus_reference_group', to='groups.group'),
        ),
        migrations.AddField(
            model_name='procedurefocusreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_focus_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='procedurefocusreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_focus_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='procedurefocusreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_focus_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='procedurefocusreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_focus_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='procedurefocusreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_focus_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='procedurefocusreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_focus_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='procedure',
            name='focus',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_focus', to='procedures.procedurefocusreference'),
        ),
        migrations.AddField(
            model_name='procedurepartofreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_partof_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='procedurepartofreference',
            name='medication_administration',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_partof_reference_medication_administration', to='medicationadministrations.medicationadministration'),
        ),
        migrations.AddField(
            model_name='procedurepartofreference',
            name='observation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_partof_reference_observation', to='observations.observation'),
        ),
        migrations.AddField(
            model_name='procedurepartofreference',
            name='procedure',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_partof_reference_procedure', to='procedures.procedure'),
        ),
        migrations.AddField(
            model_name='procedure',
            name='part_of',
            field=models.ManyToManyField(blank=True, related_name='procedure_part_of', to='procedures.procedurepartofreference'),
        ),
        migrations.AddField(
            model_name='procedureperformer',
            name='function',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_performer_function', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='procedureperformer',
            name='on_behalf_of',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_performer_on_behalf_of', to='base.organizationreference'),
        ),
        migrations.AddField(
            model_name='procedureperformer',
            name='period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_performer_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='procedure',
            name='performer',
            field=models.ManyToManyField(blank=True, related_name='procedure_performer', to='procedures.procedureperformer'),
        ),
        migrations.AddField(
            model_name='procedureperformeractorreference',
            name='care_team',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_performer_actor_reference_care_team', to='careteams.careteam'),
        ),
        migrations.AddField(
            model_name='procedureperformeractorreference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_performer_actor_reference_device', to='devices.device'),
        ),
        migrations.AddField(
            model_name='procedureperformeractorreference',
            name='healthcare_service',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_performer_actor_reference_healthcare_service', to='healthcareservices.healthcareservice'),
        ),
        migrations.AddField(
            model_name='procedureperformeractorreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_performer_actor_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='procedureperformeractorreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_performer_actor_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='procedureperformeractorreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_performer_actor_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='procedureperformeractorreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_performer_actor_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='procedureperformeractorreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_performer_actor_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='procedureperformeractorreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_performer_actor_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='procedureperformer',
            name='actor',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_performer_actor', to='procedures.procedureperformeractorreference'),
        ),
        migrations.AddField(
            model_name='procedurereasoncodeablereference',
            name='concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_reason_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='procedure',
            name='reason',
            field=models.ManyToManyField(blank=True, related_name='procedure_reason', to='procedures.procedurereasoncodeablereference'),
        ),
        migrations.AddField(
            model_name='procedurereasonreference',
            name='condition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_reason_reference_condition', to='conditions.condition'),
        ),
        migrations.AddField(
            model_name='procedurereasonreference',
            name='diagnostic_report',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_reason_reference_diagnostic_report', to='diagnosticreports.diagnosticreport'),
        ),
        migrations.AddField(
            model_name='procedurereasonreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_reason_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='procedurereasonreference',
            name='observation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_reason_reference_observation', to='observations.observation'),
        ),
        migrations.AddField(
            model_name='procedurereasonreference',
            name='procedure',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_reason_reference_procedure', to='procedures.procedure'),
        ),
        migrations.AddField(
            model_name='procedurereasoncodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_reason_codeable_reference', to='procedures.procedurereasonreference'),
        ),
        migrations.AddField(
            model_name='procedurerecorderreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_recorder_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='procedurerecorderreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_recorder_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='procedurerecorderreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_recorder_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='procedurerecorderreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_recorder_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='procedurerecorderreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_recorder_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='procedure',
            name='recorder',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_recorder', to='procedures.procedurerecorderreference'),
        ),
        migrations.AddField(
            model_name='procedurereference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='procedurereference',
            name='procedure',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_reference', to='procedures.procedure'),
        ),
        migrations.AddField(
            model_name='procedurecodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_codeable_reference_reference', to='procedures.procedurereference'),
        ),
        migrations.AddField(
            model_name='procedurereportedreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_reported_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='procedurereportedreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_reported_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='procedurereportedreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_reported_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='procedurereportedreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_reported_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='procedurereportedreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_reported_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='procedurereportedreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_reported_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='procedure',
            name='reported_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_reported_reference', to='procedures.procedurereportedreference'),
        ),
        migrations.AddField(
            model_name='procedurereportreference',
            name='diagnostic_report',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_reported_reference_diagnostic_report', to='diagnosticreports.diagnosticreport'),
        ),
        migrations.AddField(
            model_name='procedurereportreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_report_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='procedure',
            name='report',
            field=models.ManyToManyField(blank=True, related_name='procedure_report', to='procedures.procedurereportreference'),
        ),
        migrations.AddField(
            model_name='proceduresubjectreference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_subject_reference_device', to='devices.device'),
        ),
        migrations.AddField(
            model_name='proceduresubjectreference',
            name='group',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_subject_reference_group', to='groups.group'),
        ),
        migrations.AddField(
            model_name='proceduresubjectreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_subject_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='proceduresubjectreference',
            name='location',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_subject_reference_location', to='locations.location'),
        ),
        migrations.AddField(
            model_name='proceduresubjectreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_subject_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='proceduresubjectreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_subject_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='proceduresubjectreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_subject_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='procedure',
            name='subject',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_subject', to='procedures.proceduresubjectreference'),
        ),
        migrations.AddField(
            model_name='procedureusedcodeablereference',
            name='concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_used_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='procedure',
            name='used',
            field=models.ManyToManyField(blank=True, related_name='procedure_used', to='procedures.procedureusedcodeablereference'),
        ),
        migrations.AddField(
            model_name='procedureusedreference',
            name='biologically_derived_product',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_used_reference_biologically_derived_product', to='biologicallyderivedproducts.biologicallyderivedproduct'),
        ),
        migrations.AddField(
            model_name='procedureusedreference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_used_reference_device', to='devices.device'),
        ),
        migrations.AddField(
            model_name='procedureusedreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_used_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='procedureusedreference',
            name='medication',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_used_reference_medication', to='medications.medication'),
        ),
        migrations.AddField(
            model_name='procedureusedreference',
            name='substance',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_used_reference_substance', to='substances.substance'),
        ),
        migrations.AddField(
            model_name='procedureusedcodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_used_codeable_reference', to='procedures.procedureusedreference'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('clinicalimpressions', '0002_initial'),
        ('conditions', '0003_initial'),
        ('documentreferences', '0003_initial'),
        ('observations', '0002_initial'),
        ('patients', '0001_initial'),
        ('practitioners', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='clinicalimpression',
            name='performer',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='clinical_impression_performer', to='practitioners.practitionerpractitionerrolereference'),
        ),
        migrations.AddField(
            model_name='clinicalimpression',
            name='problem',
            field=models.ManyToManyField(blank=True, related_name='clinical_impression_problem', to='conditions.conditionallergyintolerancereference'),
        ),
        migrations.AddField(
            model_name='clinicalimpression',
            name='prognosis_codeable_concept',
            field=models.ManyToManyField(blank=True, related_name='clinical_impression_prognosis_codeable_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='clinicalimpression',
            name='status_reason',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='clinical_impression_status_reason', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='clinicalimpression',
            name='subject',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='clinical_impression_subject', to='patients.patientgroupreference'),
        ),
        migrations.AddField(
            model_name='clinicalimpression',
            name='supporting_info',
            field=models.ManyToManyField(blank=True, related_name='clinical_impression_supporting_info', to='base.reference'),
        ),
        migrations.AddField(
            model_name='clinicalimpression',
            name='finding',
            field=models.ManyToManyField(blank=True, related_name='clinical_impression_finding', to='clinicalimpressions.clinicalimpressionfinding'),
        ),
        migrations.AddField(
            model_name='clinicalimpressionfindingitemcodeablereference',
            name='concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='clinical_impression_finding_item_codeable_reference_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='clinicalimpressionfinding',
            name='item',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='clinical_impression_finding_item', to='clinicalimpressions.clinicalimpressionfindingitemcodeablereference'),
        ),
        migrations.AddField(
            model_name='clinicalimpressionfindingitemreference',
            name='condition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='clinical_impression_finding_item_reference_condition', to='conditions.condition'),
        ),
        migrations.AddField(
            model_name='clinicalimpressionfindingitemreference',
            name='document_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='clinical_impression_finding_item_reference_document_reference', to='documentreferences.documentreference'),
        ),
        migrations.AddField(
            model_name='clinicalimpressionfindingitemreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='clinical_impression_finding_item_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='clinicalimpressionfindingitemreference',
            name='observation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='clinical_impression_finding_item_reference_observation', to='observations.observation'),
        ),
        migrations.AddField(
            model_name='clinicalimpressionfindingitemcodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='clinical_impression_finding_item_codeable_reference_reference', to='clinicalimpressions.clinicalimpressionfindingitemreference'),
        ),
        migrations.AddField(
            model_name='clinicalimpressionreference',
            name='clinical_impression',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='clinical_impression_reference_clinical_impression', to='clinicalimpressions.clinicalimpression'),
        ),
        migrations.AddField(
            model_name='clinicalimpressionreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='clinical_impression_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='clinicalimpression',
            name='previous',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='clinical_impression_previous', to='clinicalimpressions.clinicalimpressionreference'),
        ),
    ]



"""genomic study views."""

# Create your views here.
from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.genomicstudies.models import GenomicStudy
from dfhir.genomicstudies.serializers import GenomicStudySerializer


class GenomicStudyListView(APIView):
    """GenomicStudy list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: GenomicStudySerializer(many=True)})
    def get(self, request):
        """Get a list of genomic studies."""
        genomic_studies = GenomicStudy.objects.all()
        serializer = GenomicStudySerializer(genomic_studies, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=GenomicStudySerializer, responses={201: GenomicStudySerializer}
    )
    def post(self, request):
        """Create a new genomic study."""
        serializer = GenomicStudySerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class GenomicStudyDetailView(APIView):
    """genomic study detail view."""

    def get_object(self, pk):
        """Get genomic study object."""
        try:
            return GenomicStudy.objects.get(pk=pk)
        except GenomicStudy.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: GenomicStudySerializer})
    def get(self, request, pk):
        """Get a genomic study."""
        genomic_study = self.get_object(pk)
        serializer = GenomicStudySerializer(genomic_study)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=GenomicStudySerializer, responses={200: GenomicStudySerializer}
    )
    def patch(self, request, pk):
        """Update a genomic study."""
        genomic_study = self.get_object(pk)
        serializer = GenomicStudySerializer(
            genomic_study, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk):
        """Delete a genomic study."""
        genomic_study = self.get_object(pk)
        genomic_study.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""Practitioner serializers."""

from dfhir.base.serializers import (
    AddressSerializer,
    AttachmentSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CommunicationSerializer,
    ContactPointSerializer,
    HumanNameSerializer,
    IdentifierSerializer,
    QualificationSerializer,
)

from .models import (
    Practitioner,
    PractitionerOrganizationPractitionerRoleReference,
    PractitionerPractitionerRoleReference,
    PractitionerReference,
)


class PractitionerReferenceSerializer(BaseReferenceModelSerializer):
    """Practitioner reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = PractitionerReference
        exclude = ["created_at", "updated_at"]


class PractitionerSerializer(BaseWritableNestedModelSerializer):
    """Practitioner serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    name = HumanNameSerializer(many=True, required=False)
    telecom = ContactPointSerializer(many=True, required=False)
    photo = AttachmentSerializer(many=True, required=False)
    communication = CommunicationSerializer(many=True, required=False)
    qualification = QualificationSerializer(many=True, required=False)
    address = AddressSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = Practitioner
        exclude = ["created_at", "updated_at"]


class PractitionerPractitionerRoleReferenceSerializer(BaseReferenceModelSerializer):
    """Practitioner Practitioner Role reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = PractitionerPractitionerRoleReference
        exclude = ["created_at", "updated_at"]


class PractitionerOrganizationPractitionerRoleReferenceSerializer(
    BaseReferenceModelSerializer
):
    """practitioner organization practitioner role reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = PractitionerOrganizationPractitionerRoleReference
        exclude = ["created_at", "updated_at"]



"""medications app models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    OrganizationReference,
    Quantity,
    Ratio,
    TimeStampedModel,
)
from dfhir.medications.choices import MedicationStatus


class MedicationCodes(TimeStampedModel):
    """medication codes model."""

    code = models.CharField(null=True, max_length=255)
    display = models.CharField(max_length=255, null=True)


class MedicationDoseForm(TimeStampedModel):
    """medication dose form model."""

    code = models.CharField(max_length=255, null=True)
    display = models.CharField(max_length=255, null=True)


class MedicationIngredientItem(TimeStampedModel):
    """medication ingredient item model."""

    code = models.CharField(max_length=255, null=True)
    display = models.CharField(max_length=255, null=True)


class MedicationBatch(TimeStampedModel):
    """medication batch model."""

    lot_number = models.CharField(max_length=255, null=True)
    expiration_date = models.DateTimeField(null=True)


class MedicationSubstanceReference(BaseReference):
    """medication substance reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_substance_reference_identifier",
    )
    medication = models.ForeignKey(
        "Medication",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_substance_reference",
    )
    substance = models.ForeignKey(
        "substances.Substance",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_substance_reference_substance",
    )


class MedicationSubstanceCodeableReference(TimeStampedModel):
    """medication substance codeable reference model."""

    reference = models.ForeignKey(
        MedicationSubstanceReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_substance_codeable_reference_reference",
    )
    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_substance_codeable_reference_concept",
    )


class MedicationIngredient(TimeStampedModel):
    """medication ingredient model."""

    item = models.ForeignKey(
        MedicationSubstanceCodeableReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="mediation_ingredient_item",
    )
    is_active = models.BooleanField(default=True)
    strength_ratio = models.ForeignKey(
        Ratio,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="mediation_ingredient_strength_ratio",
    )
    strength_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="mediation_ingredient_strength_codeable_concept",
    )
    strength_quantity = models.ForeignKey(
        Quantity,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="mediation_ingredient_strength_quantity",
    )


class Medication(TimeStampedModel):
    """medication model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_identifier",
    )
    code = models.ForeignKey(
        CodeableConcept,
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="medication_code",
    )
    status = models.CharField(
        max_length=255, choices=MedicationStatus.choices, null=True
    )
    marketing_authorization_holder = models.ForeignKey(
        OrganizationReference,
        on_delete=models.DO_NOTHING,
        related_name="medication_organization",
        null=True,
    )
    dose_form = models.OneToOneField(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_dose_form",
    )
    total_volume = models.ForeignKey(
        Quantity,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_total_volume",
    )
    ingredient = models.ManyToManyField(
        MedicationIngredient,
        blank=True,
        related_name="medication_ingredient",
    )
    batch = models.ForeignKey(
        MedicationBatch,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_batch",
    )
    definition = models.ForeignKey(
        "medicationknowledges.MedicationKnowledgeReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_definition",
    )


class MedicationReference(BaseReference):
    """medication reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_reference_identifier",
    )
    medication = models.ForeignKey(
        Medication,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_reference_medication",
    )


class MedicationCodeableReference(TimeStampedModel):
    """medication codeable reference model."""

    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_codeable_reference_concept",
    )
    reference = models.ForeignKey(
        MedicationReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_codeable_reference_reference",
    )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('conditions', '0004_initial'),
        ('devicerequests', '0003_initial'),
        ('devices', '0001_initial'),
        ('diagnosticreports', '0002_initial'),
        ('documentreferences', '0003_initial'),
        ('groups', '0001_initial'),
        ('locations', '0001_initial'),
        ('observations', '0002_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0001_initial'),
        ('practitionerroles', '0001_initial'),
        ('practitioners', '0001_initial'),
        ('relatedpersons', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='devicerequestperformerreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_performer_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='devicerequestperformerreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_performer_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='devicerequestperformerreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_performer_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='devicerequestperformercodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_performer_codeable_reference_reference', to='devicerequests.devicerequestperformerreference'),
        ),
        migrations.AddField(
            model_name='devicerequestreasoncodeablereference',
            name='concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_reason_codeable_reference_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='devicerequest',
            name='reason',
            field=models.ManyToManyField(blank=True, related_name='device_request_reason', to='devicerequests.devicerequestreasoncodeablereference'),
        ),
        migrations.AddField(
            model_name='devicerequestreasonreference',
            name='condition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_reason_reference_condition', to='conditions.condition'),
        ),
        migrations.AddField(
            model_name='devicerequestreasonreference',
            name='diagnostic_report',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_reason_reference_diagnostic_report', to='diagnosticreports.diagnosticreport'),
        ),
        migrations.AddField(
            model_name='devicerequestreasonreference',
            name='document_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_reason_reference_document_reference', to='documentreferences.documentreference'),
        ),
        migrations.AddField(
            model_name='devicerequestreasonreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_reason_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='devicerequestreasonreference',
            name='observation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_reason_reference_observation', to='observations.observation'),
        ),
        migrations.AddField(
            model_name='devicerequestreasoncodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_reason_codeable_reference_reference', to='devicerequests.devicerequestreasonreference'),
        ),
        migrations.AddField(
            model_name='devicerequestreference',
            name='device_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_reference_device_request', to='devicerequests.devicerequest'),
        ),
        migrations.AddField(
            model_name='devicerequestreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='devicerequest',
            name='replaces',
            field=models.ManyToManyField(blank=True, related_name='device_request_replaces', to='devicerequests.devicerequestreference'),
        ),
        migrations.AddField(
            model_name='devicerequestrequestercodeablereference',
            name='concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_requester_codeable_reference_code', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='devicerequestrequesterreference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_requester_reference_device', to='devices.device'),
        ),
        migrations.AddField(
            model_name='devicerequestrequesterreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_requester_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='devicerequestrequesterreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_requester_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='devicerequestrequesterreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_requester_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='devicerequestrequesterreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_requester_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='devicerequestrequestercodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_requester_codeable_reference_reference', to='devicerequests.devicerequestrequesterreference'),
        ),
        migrations.AddField(
            model_name='devicerequest',
            name='requester',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_requester', to='devicerequests.devicerequestrequesterreference'),
        ),
        migrations.AddField(
            model_name='devicerequestsubjectreference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_subject_reference_device', to='devices.device'),
        ),
        migrations.AddField(
            model_name='devicerequestsubjectreference',
            name='group',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_subject_reference_group', to='groups.group'),
        ),
        migrations.AddField(
            model_name='devicerequestsubjectreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_subject_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='devicerequestsubjectreference',
            name='location',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_subject_reference_location', to='locations.location'),
        ),
        migrations.AddField(
            model_name='devicerequestsubjectreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_subject_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='devicerequest',
            name='subject',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_subject', to='devicerequests.devicerequestsubjectreference'),
        ),
    ]



"""medication model choices."""

from django.db.models import TextChoices
from django.utils.translation import gettext_lazy as _


class MedicationStatus(TextChoices):
    """medication status choices."""

    ACTIVE = "active", _("Active")
    INACTIVE = "inactive", _("Inactive")
    ENTERED_IN_ERROR = "entered_in_error", _("Entered in error")


class MedicationIngredientStrength(TextChoices):
    """medication ingredient strength choices."""

    SUFFICIENT = "sufficient", _("Sufficient")
    SMALL_AMOUNT = "small_amount", _("Small amount")


class DoseAdministrationAid(TextChoices):
    """dose administration aid."""

    BLISTER_PACK = "blister_pack", _("Blister pack")
    DOSETTE = "dosette", _("Dosette")
    SACHETS = "sachets", _("Sachets")


class SubstitutionReason(TextChoices):
    """substitution reason."""

    CONTINUING_THERAPY = "continuing_therapy", _("Continuing therapy")
    FORMULARY_POLICY = "formulary_policy", _("Formulary policy")
    OUT_OF_STOCK = "out of stock", _("Out of stock")
    REGULATORY_REQUIREMENTS = "regulatory_requirements", _("Regulatory requirements")



"""Personal Relationships App Configuration."""

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class PersonalrelationshipsConfig(AppConfig):
    """Personal Relationships Config."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "dfhir.personalrelationships"
    verbose_name = _("Personal Relationships")



"""imaging study model choices."""

from django.db.models import TextChoices
from django.utils.translation import gettext_lazy as _


class ImagingStudyStatusChoices(TextChoices):
    """Imaging study status choices."""

    REGISTERED = "registered", _("Registered")
    AVAILABLE = "available", _("Available")
    CANCELLED = "cancelled", _("Cancelled")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in error")
    UNKNOWN = "unknown", _("Unknown")
    INACTIVE = "inactive", _("Inactive")



"""imaging study views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.imagingstudies.models import ImagingStudy
from dfhir.imagingstudies.serializers import ImagingStudySerializer


class ImagingStudyListView(APIView):
    """imaging Study list view."""

    permission_classes = (AllowAny,)

    @extend_schema(responses={200: ImagingStudySerializer(many=True)})
    def get(self, request):
        """Get imaging study list."""
        imaging_studies = ImagingStudy.objects.all()
        serializer = ImagingStudySerializer(imaging_studies, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=ImagingStudySerializer, responses={201: ImagingStudySerializer}
    )
    def post(self, request):
        """Create a new imaging study."""
        serializer = ImagingStudySerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class ImagingStudyDetailView(APIView):
    """imaging study detail view."""

    permission_classes = (AllowAny,)

    def get_object(self, pk):
        """Get imaging study detail."""
        try:
            return ImagingStudy.objects.get(pk=pk)
        except ImagingStudy.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: ImagingStudySerializer})
    def get(self, request, pk):
        """Get imaging study detail."""
        imaging_study = self.get_object(pk)
        serializer = ImagingStudySerializer(imaging_study)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={200: ImagingStudySerializer})
    def patch(self, request, pk):
        """Update imaging study detail."""
        imaging_study = self.get_object(pk)
        serializer = ImagingStudySerializer(
            imaging_study, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={200: ImagingStudySerializer})
    def delete(self, request, pk):
        """Delete imaging study detail."""
        imaging_study = self.get_object(pk)
        imaging_study.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""provenance serializers."""

from drf_writable_nested import WritableNestedModelSerializer

from dfhir.base.serializers import (
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    ReferenceSerializer,
)
from dfhir.provenances.models import (
    Provenance,
    ProvenanceAgent,
    ProvenanceAgentOnBehalfOf,
    ProvenanceAgentWhoReference,
    ProvenanceEntry,
    ProvenanceReference,
)


class ProvenanceWhoReferenceSerializer(BaseReferenceModelSerializer):
    """provenance who reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = ProvenanceAgentWhoReference
        exclude = ["created_at", "updated_at"]


class ProvenanceAgentOnBehalfOfSerializer(BaseReferenceModelSerializer):
    """Provenance agent on behalf of serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ProvenanceAgentOnBehalfOf
        exclude = ["created_at", "updated_at"]


class ProvenanceAgentSerializer(WritableNestedModelSerializer):
    """provenance agent serializer."""

    type = CodeableConceptSerializer(required=False)
    role = CodeableConceptSerializer(required=False)
    who = ProvenanceWhoReferenceSerializer(many=False, required=False)
    on_behalf_of = ProvenanceAgentOnBehalfOfSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ProvenanceAgent
        exclude = ["created_at", "updated_at"]


class ProvenanceEntrySerializer(WritableNestedModelSerializer):
    """provenance entry serializer."""

    what = ReferenceSerializer(many=False, required=False)
    agent = ProvenanceAgentSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = ProvenanceEntry
        exclude = ["created_at", "updated_at"]


class ProvenanceReferenceSerializer(BaseReferenceModelSerializer):
    """Provenance reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ProvenanceReference
        exclude = ["created_at", "updated_at"]


class ProvenanceSerializer(BaseWritableNestedModelSerializer):
    """Provenance serializer."""

    target = ProvenanceReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = Provenance
        exclude = ["created_at", "updated_at"]



"""EnrollmentResponses models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    Identifier,
    OrganizationReference,
    TimeStampedModel,
)

from . import choices


class EnrollmentResponseRequestProviderReference(BaseReference):
    """EnrollmentResponse request provider reference model."""

    identifier = models.ForeignKey(
        Identifier,
        related_name="enrollment_response_request_provider_reference_identifier",
        on_delete=models.CASCADE,
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        related_name="enrollment_response_request_provider_reference_practitioner",
        on_delete=models.CASCADE,
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        related_name="enrollment_response_request_provider_reference_practitioner_role",
        on_delete=models.CASCADE,
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        related_name="enrollment_response_request_provider_reference_organization",
        on_delete=models.CASCADE,
        null=True,
    )


class EnrollmentResponse(TimeStampedModel):
    """EnrollmentResponse model."""

    identifier = models.ManyToManyField(
        Identifier,
        related_name="enrollment_response_identifier",
        blank=True,
    )
    status = models.CharField(
        max_length=50,
        choices=choices.EnrollmentResponseStatus.choices,
        null=True,
    )
    request = models.ForeignKey(
        "enrollmentrequests.EnrollmentRequest",
        related_name="enrollment_response_request",
        on_delete=models.CASCADE,
        null=True,
    )
    outcome = models.CharField(
        max_length=50,
        choices=choices.EnrollmentResponseOutcome.choices,
        null=True,
    )
    disposition = models.TextField(null=True)
    created = models.DateTimeField(auto_created=True, null=True)
    organization = models.ForeignKey(
        OrganizationReference,
        related_name="enrollment_response_organization",
        on_delete=models.CASCADE,
        null=True,
    )
    request_provider = models.ForeignKey(
        EnrollmentResponseRequestProviderReference,
        related_name="enrollment_response_request_provider",
        on_delete=models.CASCADE,
        null=True,
    )



"""care plan views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import CarePlan
from .serializers import CarePlanSerializer


class CarePlanListView(APIView):
    """CarePlan list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: CarePlanSerializer(many=True)})
    def get(self, request):
        """Get a list of care plans."""
        care_plans = CarePlan.objects.all()
        serializer = CarePlanSerializer(care_plans, many=True)
        return Response(serializer.data)

    @extend_schema(request=CarePlanSerializer, responses={201: CarePlanSerializer})
    def post(self, request):
        """Create a care plan."""
        serializer = CarePlanSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class CarePlanDetailView(APIView):
    """CarePlan detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get a care plan object."""
        try:
            return CarePlan.objects.get(pk=pk)
        except CarePlan.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: CarePlanSerializer})
    def get(self, request, pk=None):
        """Get a care plan."""
        care_plan = self.get_object(pk)
        serializer = CarePlanSerializer(care_plan)
        return Response(serializer.data)

    @extend_schema(request=CarePlanSerializer, responses={200: CarePlanSerializer})
    def patch(self, request, pk=None):
        """Update a care plan."""
        care_plan = self.get_object(pk)
        serializer = CarePlanSerializer(care_plan, data=request.data, partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)

    @extend_schema(responses={204: None})
    def delete(self, request, pk=None):
        """Delete a care plan."""
        care_plan = self.get_object(pk)
        care_plan.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""Episode of care urls."""

from django.urls import path

from . import views

app_name = "episodeofcares"

urlpatterns = [
    path("episodeofcares/", views.EpisodeOfCareListView.as_view()),
    path("episodeofcares/<int:pk>/", views.EpisodeOfCareDetailView.as_view()),
]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('bodystructures', '0002_initial'),
        ('claims', '0004_initial'),
        ('conditions', '0004_initial'),
        ('devices', '0001_initial'),
        ('deviceusages', '0001_initial'),
        ('diagnosticreports', '0002_initial'),
        ('documentreferences', '0004_initial'),
        ('encounters', '0003_initial'),
        ('observations', '0002_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0002_initial'),
        ('practitionerroles', '0002_initial'),
        ('practitioners', '0001_initial'),
        ('procedures', '0001_initial'),
        ('relatedpersons', '0001_initial'),
        ('servicerequests', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='deviceusage',
            name='based_on',
            field=models.ManyToManyField(blank=True, related_name='device_usage_based_on', to='servicerequests.servicerequestreference'),
        ),
        migrations.AddField(
            model_name='deviceusage',
            name='body_site',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_body_site', to='bodystructures.bodystructurecodeablereference'),
        ),
        migrations.AddField(
            model_name='deviceusage',
            name='category',
            field=models.ManyToManyField(blank=True, related_name='device_usage_category', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='deviceusage',
            name='context',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_context', to='encounters.encounterepisodeofcarereference'),
        ),
        migrations.AddField(
            model_name='deviceusage',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_device', to='devices.devicedevicedefinitioncodeablereference'),
        ),
        migrations.AddField(
            model_name='deviceusage',
            name='identifier',
            field=models.ManyToManyField(blank=True, related_name='device_usages_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='deviceusage',
            name='note',
            field=models.ManyToManyField(blank=True, related_name='device_usage_note', to='base.annotation'),
        ),
        migrations.AddField(
            model_name='deviceusage',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_patient', to='patients.patientreference'),
        ),
        migrations.AddField(
            model_name='deviceusage',
            name='timing_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_timing_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='deviceusage',
            name='timing_timing',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_timing_timing', to='base.timing'),
        ),
        migrations.AddField(
            model_name='deviceusage',
            name='usage_reason',
            field=models.ManyToManyField(blank=True, related_name='device_usage_usage_reason', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='deviceusage',
            name='usage_status',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_usage_status', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='deviceusageadherence',
            name='code',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_adherence_code', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='deviceusageadherence',
            name='reason',
            field=models.ManyToManyField(blank=True, related_name='device_usage_adherence_reason', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='deviceusage',
            name='adherence',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_adherence', to='deviceusages.deviceusageadherence'),
        ),
        migrations.AddField(
            model_name='deviceusagederivedfromreference',
            name='claim',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_derived_from_reference_claim', to='claims.claim'),
        ),
        migrations.AddField(
            model_name='deviceusagederivedfromreference',
            name='document_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_derived_from_reference_document_reference', to='documentreferences.documentreference'),
        ),
        migrations.AddField(
            model_name='deviceusagederivedfromreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_derived_from_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='deviceusagederivedfromreference',
            name='observation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_derived_from_reference_observation', to='observations.observation'),
        ),
        migrations.AddField(
            model_name='deviceusagederivedfromreference',
            name='procedure',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_derived_from_reference_procedure', to='procedures.procedure'),
        ),
        migrations.AddField(
            model_name='deviceusagederivedfromreference',
            name='service_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_derived_from_reference_service_request', to='servicerequests.servicerequest'),
        ),
        migrations.AddField(
            model_name='deviceusage',
            name='derived_from',
            field=models.ManyToManyField(blank=True, related_name='device_usage_derived_from', to='deviceusages.deviceusagederivedfromreference'),
        ),
        migrations.AddField(
            model_name='deviceusageinformationsourcereference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_usage_information_source_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='deviceusageinformationsourcereference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_information_source_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='deviceusageinformationsourcereference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_information_source_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='deviceusageinformationsourcereference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_information_source_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='deviceusageinformationsourcereference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_information_source_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='deviceusageinformationsourcereference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_information_source_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='deviceusage',
            name='information_source',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_information_source', to='deviceusages.deviceusageinformationsourcereference'),
        ),
        migrations.AddField(
            model_name='deviceusagereasoncodeablereference',
            name='concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_usage_reason_codeable_reference_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='deviceusage',
            name='reason',
            field=models.ManyToManyField(blank=True, related_name='device_usage_reason', to='deviceusages.deviceusagereasoncodeablereference'),
        ),
        migrations.AddField(
            model_name='deviceusagereasonreference',
            name='condition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_reason_reference_condition', to='conditions.condition'),
        ),
        migrations.AddField(
            model_name='deviceusagereasonreference',
            name='diagnostic_report',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_reason_reference_diagnostic_report', to='diagnosticreports.diagnosticreport'),
        ),
        migrations.AddField(
            model_name='deviceusagereasonreference',
            name='document_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_reason_reference_document_reference', to='documentreferences.documentreference'),
        ),
        migrations.AddField(
            model_name='deviceusagereasonreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_reason_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='deviceusagereasonreference',
            name='observation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_reason_reference_observation', to='observations.observation'),
        ),
        migrations.AddField(
            model_name='deviceusagereasonreference',
            name='procedure',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_usage_reason_reference_procedure', to='procedures.procedure'),
        ),
        migrations.AddField(
            model_name='deviceusagereasoncodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_usage_reason_codeable_reference_reference', to='deviceusages.deviceusagereasonreference'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('bodystructures', '0001_initial'),
        ('devices', '0001_initial'),
        ('imagingselections', '0002_initial'),
        ('patients', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='bodystructure',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='body_structure_patient', to='patients.patientreference'),
        ),
        migrations.AddField(
            model_name='bodystructurecodeablereference',
            name='concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='bodystructurereference',
            name='body_structure',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='bodystructures.bodystructure'),
        ),
        migrations.AddField(
            model_name='bodystructurereference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='base.identifier'),
        ),
        migrations.AddField(
            model_name='bodystructurecodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='bodystructures.bodystructurereference'),
        ),
        migrations.AddField(
            model_name='distancefromlandmark',
            name='device',
            field=models.ManyToManyField(blank=True, related_name='distance_from_landmark_device', to='devices.devicecodeablereference'),
        ),
        migrations.AddField(
            model_name='distancefromlandmark',
            name='value',
            field=models.ManyToManyField(blank=True, related_name='distance_from_landmark_value', to='base.quantity'),
        ),
        migrations.AddField(
            model_name='bodylandmarkorientation',
            name='distance_from_landmark',
            field=models.ManyToManyField(blank=True, related_name='body_landmark_orientation_distance_from_landmark', to='bodystructures.distancefromlandmark'),
        ),
        migrations.AddField(
            model_name='includedstructure',
            name='body_landmark_orientation',
            field=models.ManyToManyField(blank=True, related_name='included_structure_body_landmark_orientation', to='bodystructures.bodylandmarkorientation'),
        ),
        migrations.AddField(
            model_name='includedstructure',
            name='laterality',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='included_structure_laterality', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='includedstructure',
            name='qualifier',
            field=models.ManyToManyField(blank=True, related_name='included_structure_qualifier', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='includedstructure',
            name='spatial_reference',
            field=models.ManyToManyField(blank=True, related_name='included_structure_spatial_reference', to='imagingselections.imagingselectionreference'),
        ),
        migrations.AddField(
            model_name='includedstructure',
            name='structure',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='included_structure_structure', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='bodystructure',
            name='excluded_structure',
            field=models.ManyToManyField(blank=True, related_name='body_structure_excluded_structure', to='bodystructures.includedstructure'),
        ),
        migrations.AddField(
            model_name='bodystructure',
            name='included_structure',
            field=models.ManyToManyField(blank=True, related_name='body_structure_included_structure', to='bodystructures.includedstructure'),
        ),
    ]



"""Base serializers."""

import inspect
from typing import Any, TypeVar

from drf_writable_nested.serializers import WritableNestedModelSerializer
from rest_framework import serializers

from . import utils
from .models import (
    Address,
    Age,
    Annotation,
    Attachment,
    Availability,
    AvailableTime,
    CodeableConcept,
    CodeableReference,
    Coding,
    Communication,
    ContactDetail,
    ContactPoint,
    Duration,
    Expression,
    ExtendedContactDetail,
    HumanName,
    Identifier,
    MonetaryComponent,
    Money,
    NotAvailableTime,
    OrganizationReference,
    Payload,
    Period,
    ProductShelfLife,
    Qualification,
    Quantity,
    Range,
    Ratio,
    Reference,
    RelatedArtifact,
    RelativeTime,
    Repeat,
    Signature,
    SignatureOnBehalfOfReference,
    SignatureWhoReference,
    SimpleQuantity,
    Timing,
    TriggerDefinition,
    UsageContext,
    VirtualServiceDetailAddress,
    VirtualServiceDetails,
)

_VT = TypeVar("_VT")
_DT = TypeVar("_DT")
_IN = TypeVar("_IN")


class RenameFieldsMixin(serializers.ModelSerializer):
    """Rename fields mixin."""

    def __init__(self, *args, **kwargs):
        """Initialize the serializer and set up field renaming."""
        super().__init__(*args, **kwargs)
        self.rename_fields = getattr(self.Meta, "rename_fields", {})

    def to_representation(self, instance: _IN) -> dict[str, Any]:
        """Rename fields in the output data."""
        data = super().to_representation(instance)
        for new_field, original_field in self.rename_fields.items():
            if original_field in data:
                data[new_field] = data.pop(original_field)
        return data

    def to_internal_value(self, data: _DT) -> _VT:
        """Rename fields in the input data."""
        for new_field, original_field in self.rename_fields.items():
            if new_field in data:
                data[original_field] = data.pop(new_field)

        return super().to_internal_value(data)

    def get_fields(self) -> dict[str, serializers.Field]:
        """Rename fields in the serializer definition."""
        for new_field, original_field in self.rename_fields.items():
            if new_field in self._declared_fields:
                self._declared_fields[original_field] = self._declared_fields.pop(
                    new_field
                )
        return super().get_fields()


class BaseModelSerializer(RenameFieldsMixin, serializers.ModelSerializer):
    """Base model serializer."""

    def get_fields(self) -> dict[str, serializers.Field]:
        """Get fields."""
        fields = super().get_fields()
        fields["resource_type"] = serializers.ReadOnlyField()
        return fields


class BaseWritableNestedModelSerializer(
    RenameFieldsMixin, WritableNestedModelSerializer
):
    """Base writable nested model serializer."""

    def get_fields(self) -> dict[str, serializers.Field]:
        """Get fields."""
        fields = super().get_fields()
        fields["resource_type"] = serializers.ReadOnlyField()
        return fields


class BaseReferenceModelSerializer(WritableNestedModelSerializer):
    """Base reference model serializer."""

    def to_internal_value(self, data: _DT) -> _VT:
        """Convert reference to internal value."""
        try:
            ref = data.get("reference")
            if ref and isinstance(ref, str):
                key, value = ref.split("/")
                data[utils.to_snake_case(key)] = int(value)
        except AttributeError as e:
            raise AttributeError(
                f"{e} |  ModelClass: {self.Meta.model.__name__}  -  serializerClass: {self.__class__.__name__} -  data: {data}"
            ) from e
        except Exception as e:
            raise serializers.ValidationError(
                f"Invalid reference: {ref}. References should be in the format '<ResourceName>/[id]'"
            ) from e
        return super().to_internal_value(data)


class OrganizationReferenceSerializer(BaseReferenceModelSerializer):
    """Organization reference serializer."""

    def get_fields(self):
        """Get fields."""
        fields = super().get_fields()
        frames = inspect.stack()

        for frame in frames:
            if "drf_spectacular" in frame.filename:
                # when generating docs using drf_spectacular, skip the identifier field
                # to avoid a recursive loop.
                return fields

        fields["identifier"] = IdentifierSerializer(required=False, many=False)
        return fields

    class Meta:
        """Meta class."""

        model = OrganizationReference
        exclude = [
            "created_at",
            "updated_at",
        ]


class CodingSerializer(serializers.ModelSerializer):
    """Coding serializer."""

    class Meta:
        """Meta class."""

        model = Coding
        exclude = ["created_at", "updated_at"]


class CodeableConceptSerializer(WritableNestedModelSerializer):
    """CodeableConcept serializer."""

    coding = CodingSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = CodeableConcept
        exclude = ["created_at", "updated_at"]


class PeriodSerializer(serializers.ModelSerializer):
    """Period serializer."""

    class Meta:
        """Meta class."""

        model = Period
        exclude = ["created_at", "updated_at"]


class IdentifierSerializer(WritableNestedModelSerializer):
    """Identifier serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    period = PeriodSerializer(many=False, required=False)
    assigner = OrganizationReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = Identifier
        exclude = ["created_at", "updated_at"]


class QualificationSerializer(WritableNestedModelSerializer):
    """Qualification serializer."""

    code = CodeableConceptSerializer(many=False, required=False)
    period = PeriodSerializer(many=False, required=False)
    identifier = IdentifierSerializer(many=True, required=False)
    issuer = OrganizationReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = Qualification
        exclude = ["created_at", "updated_at"]


class AddressSerializer(WritableNestedModelSerializer):
    """Address serializer."""

    period = PeriodSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = Address
        exclude = ["created_at", "updated_at"]


class HumanNameSerializer(WritableNestedModelSerializer):
    """HumanName serializer."""

    period = PeriodSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = HumanName
        exclude = ["created_at", "updated_at"]


class ContactPointSerializer(WritableNestedModelSerializer):
    """ContactPoint serializer."""

    period = PeriodSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ContactPoint
        exclude = ["created_at", "updated_at"]


class ExtendedContactDetailSerializer(WritableNestedModelSerializer):
    """ExtendedContactDetail serializer."""

    name = HumanNameSerializer(many=True, required=False)
    address = AddressSerializer(many=False, required=False)
    period = PeriodSerializer(many=False, required=False)
    telecom = ContactPointSerializer(many=True, required=False)
    purpose = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ExtendedContactDetail
        exclude = ["created_at", "updated_at"]


class PayloadSerializer(WritableNestedModelSerializer):
    """Payload serializer."""

    type = CodeableConceptSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = Payload
        exclude = ["created_at", "updated_at"]


class AvailableTimeSerializer(serializers.ModelSerializer):
    """AvailableTime serializer."""

    class Meta:
        """Meta class."""

        model = AvailableTime
        exclude = ["created_at", "updated_at"]


class NotAvailableTimeSerializer(WritableNestedModelSerializer):
    """NotAvailableTime serializer."""

    during = PeriodSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = NotAvailableTime
        exclude = ["created_at", "updated_at"]


class AvailabilitySerializer(WritableNestedModelSerializer):
    """Availability serializer."""

    period = PeriodSerializer(many=True, required=False)
    available_time = AvailableTimeSerializer(many=True, required=False)
    not_available_time = NotAvailableTimeSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = Availability
        exclude = ["created_at", "updated_at"]


class VirtualServiceDetailAddressSerializer(WritableNestedModelSerializer):
    """VirtualServiceDetailAddress serializer."""

    address_contact_point = ContactPointSerializer(many=True, required=False)
    address_extended_contact_detail = ExtendedContactDetailSerializer(
        many=True, required=False
    )

    class Meta:
        """Meta class."""

        model = VirtualServiceDetailAddress
        exclude = ["created_at", "updated_at"]


class VirtualServiceDetailsSerializer(WritableNestedModelSerializer):
    """VirtualServiceDetails serializer."""

    channel_type = CodingSerializer(many=False, required=False)
    address = VirtualServiceDetailAddressSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = VirtualServiceDetails
        exclude = ["created_at", "updated_at"]


class AttachmentSerializer(WritableNestedModelSerializer):
    """Attachment serializer."""

    class Meta:
        """Meta class."""

        model = Attachment
        exclude = ["created_at", "updated_at"]


class QuantitySerializer(serializers.ModelSerializer):
    """Quantity serializer."""

    class Meta:
        """Meta class."""

        model = Quantity
        exclude = ["created_at", "updated_at"]


class RangeSerializer(WritableNestedModelSerializer):
    """Range serializer."""

    low = QuantitySerializer(many=False, required=False)
    high = QuantitySerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = Range
        exclude = ["created_at", "updated_at"]


class ReferenceSerializer(serializers.ModelSerializer):
    """Reference serializer."""

    identifier = IdentifierSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = Reference
        exclude = ["created_at", "updated_at"]


class AnnotationSerializer(WritableNestedModelSerializer):
    """Annotation serializer."""

    author_reference = ReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = Annotation
        exclude = ["created_at", "updated_at"]


class CommunicationSerializer(WritableNestedModelSerializer):
    """Communication serializer."""

    language = CodeableConceptSerializer(required=False)

    class Meta:
        """Meta class."""

        model = Communication
        exclude = ["created_at", "updated_at"]


class RatioSerializer(WritableNestedModelSerializer):
    """Ratio serializer."""

    numerator = QuantitySerializer(many=False, required=False)
    denominator = QuantitySerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = Ratio
        exclude = ["created_at", "updated_at"]


class CodeableReferenceSerializer(WritableNestedModelSerializer):
    """recommendation codeable reference serializer."""

    reference = ReferenceSerializer(many=False, required=False)
    concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = CodeableReference
        exclude = ["created_at", "updated_at"]


class RepeatSerializer(WritableNestedModelSerializer):
    """Repeat serializer."""

    bounds_duration = QuantitySerializer(many=False, required=False)
    bounds_range = RangeSerializer(many=False, required=False)
    bounds_period = PeriodSerializer(many=False, required=False)
    day_of_week = CodingSerializer(many=True, required=False)
    when = CodingSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = Repeat
        exclude = ["created_at", "updated_at"]


class TimingSerializer(WritableNestedModelSerializer):
    """timing serializer."""

    repeat = RepeatSerializer(many=False, required=False)
    code = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = Timing
        exclude = ["created_at", "updated_at"]


class RelatedArtifactSerializer(WritableNestedModelSerializer):
    """RelatedArtifact serializer."""

    classifier = CodeableConceptSerializer(many=False, required=False)
    document = AttachmentSerializer(many=False, required=False)
    resource_reference = ReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = RelatedArtifact
        exclude = ["created_at", "updated_at"]


class ProductShelfLifeSerializer(WritableNestedModelSerializer):
    """ProductShelfLife serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    period_duration = QuantitySerializer(many=False, required=False)
    special_precautions_for_storage = CodeableConceptSerializer(
        many=False, required=False
    )

    class Meta:
        """Meta class."""

        model = ProductShelfLife
        exclude = ["created_at", "updated_at"]


class UsageContextSerializer(WritableNestedModelSerializer):
    """UsageContext serializer."""

    code = CodingSerializer(many=False, required=False)
    value_codeable_concept = CodeableConceptSerializer(many=False, required=False)
    value_quantity = QuantitySerializer(many=False, required=False)
    value_range = RangeSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = UsageContext
        exclude = ["created_at", "updated_at"]


class ContactDetailSerializer(WritableNestedModelSerializer):
    """ContactDetail serializer."""

    telecom = ContactPointSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = ContactDetail
        exclude = ["created_at", "updated_at"]


class ExpressionSerializer(WritableNestedModelSerializer):
    """Expression serializer."""

    class Meta:
        """Meta class."""

        model = Expression
        exclude = ["created_at", "updated_at"]


class RelativeTimeSerializer(WritableNestedModelSerializer):
    """RelativeTime serializer."""

    context_reference = ReferenceSerializer(many=False, required=False)
    context_codeable_concept = CodeableConceptSerializer(many=False, required=False)
    offset_duration = QuantitySerializer(many=False, required=False)
    offset_range = RangeSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = RelativeTime
        exclude = ["created_at", "updated_at"]


class AgeSerializer(QuantitySerializer):
    """Age serializer."""

    class Meta:
        """Meta class."""

        model = Age
        exclude = ["created_at", "updated_at"]


class SimpleQuantitySerializer(QuantitySerializer):
    """SimpleQuantity serializer."""

    class Meta:
        """Meta class."""

        model = SimpleQuantity
        exclude = ["created_at", "updated_at"]


class DurationSerializer(QuantitySerializer):
    """Duration serializer."""

    class Meta:
        """Meta class."""

        model = Duration
        exclude = ["created_at", "updated_at"]


class MoneySerializer(serializers.ModelSerializer):
    """Money serializer."""

    class Meta:
        """Meta class."""

        model = Money
        exclude = ["created_at", "updated_at"]


class MonetaryComponentSerializer(WritableNestedModelSerializer):
    """MonetaryComponent serializer."""

    code = CodeableConceptSerializer(many=False, required=False)
    amount = MoneySerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = MonetaryComponent
        exclude = ["created_at", "updated_at"]


class SignatureWhoReferenceSerializer(BaseReferenceModelSerializer):
    """Signature who reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = SignatureWhoReference
        exclude = ["created_at", "updated_at"]


class SignatureOnBehalfOfReferenceSerializer(BaseReferenceModelSerializer):
    """Signature on behalf of reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = SignatureOnBehalfOfReference
        exclude = ["created_at", "updated_at"]


class SignatureSerializer(WritableNestedModelSerializer):
    """Signature serializer."""

    type = CodingSerializer(many=True, required=False)
    who = SignatureWhoReferenceSerializer(many=False, required=False)
    on_behalf_of = SignatureOnBehalfOfReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = Signature
        exclude = ["created_at", "updated_at"]


class TriggerDefinitionSerializer(WritableNestedModelSerializer):
    """TriggerDefinition serializer."""

    code = CodeableConceptSerializer(many=False, required=False)
    timing_timing = TimingSerializer(many=False, required=False)
    condition = ExpressionSerializer(many=False, required=False)

    def get_fields(self) -> dict[str, serializers.Field]:
        """Get fields."""
        from dfhir.schedules.serializers import ScheduleReferenceSerializer

        fields = super().get_fields()

        fields["timing_reference"] = ScheduleReferenceSerializer(
            required=False, many=False
        )
        return fields

    class Meta:
        """Meta class."""

        model = TriggerDefinition
        exclude = ["created_at", "updated_at"]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0004_initial'),
        ('healthcareservices', '0001_initial'),
        ('patients', '0001_initial'),
        ('practitioners', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Schedule',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('active', models.BooleanField(default=True)),
                ('name', models.CharField(max_length=255, null=True)),
                ('slots_duration', models.IntegerField(default=30)),
                ('comment', models.TextField(null=True)),
                ('identifier', models.ManyToManyField(blank=True, related_name='schedule_identifier', to='base.identifier')),
                ('planning_horizon', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='base.period')),
                ('service_category', models.ManyToManyField(related_name='schedule_category', to='base.codeableconcept')),
                ('service_type', models.ManyToManyField(related_name='schedule_servicetype', to='healthcareservices.healthcareservicecodeablereference')),
                ('specialty', models.ManyToManyField(related_name='schedule_specialty', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ScheduleReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='schedule_reference_identifier', to='base.identifier')),
                ('schedule', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='schedule_reference_schedule', to='schedules.schedule')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SchedulesActorReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('healthcareservice', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='schedules_actor_healthcareservice', to='healthcareservices.healthcareservice')),
                ('identifier', models.ManyToManyField(blank=True, related_name='schedules_actor_identifier', to='base.identifier')),
                ('patient', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='schedules_actor_patient', to='patients.patient')),
                ('practitioner', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='schedules_actor_practitioner', to='practitioners.practitioner')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='schedule',
            name='actor',
            field=models.ManyToManyField(related_name='schedule_actor', to='schedules.schedulesactorreference'),
        ),
    ]



"""Specimen definition serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    CodingSerializer,
    ContactDetailSerializer,
    IdentifierSerializer,
    PeriodSerializer,
    QuantitySerializer,
    RangeSerializer,
    UsageContextSerializer,
)
from dfhir.groups.serializers import GroupReferenceSerializer

from .models import (
    SpecimenDefinition,
    SpecimenDefinitionReference,
    SpecimenDefinitionTypeTested,
    SpecimenDefinitionTypeTestedContainer,
    SpecimenDefinitionTypeTestedContainerAdditive,
    SpecimenDefinitionTypeTestedHandling,
)


class SpecimenDefinitionReferenceSerializer(BaseReferenceModelSerializer):
    """Serializer for specimen definition reference."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = SpecimenDefinitionReference
        exclude = ["created_at", "updated_at"]


class SpecimenDefinitionTypeTestedContainerAdditiveSerializer(
    WritableNestedModelSerializer
):
    """Serializer for specimen definition type tested container additive."""

    additive_codeable_concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = SpecimenDefinitionTypeTestedContainerAdditive
        exclude = ["created_at", "updated_at"]


class SpecimenDefinitionTypeTestedContainerSerializer(WritableNestedModelSerializer):
    """Serializer for specimen definition type tested container."""

    material = CodeableConceptSerializer(many=False, required=False)
    type = CodeableConceptSerializer(many=False, required=False)
    cap = CodeableConceptSerializer(many=False, required=False)
    capacity = QuantitySerializer(many=False, required=False)
    minimum_volume_quantity = QuantitySerializer(many=False, required=False)
    additive = SpecimenDefinitionTypeTestedContainerAdditiveSerializer(
        many=True, required=False
    )

    class Meta:
        """meta options."""

        model = SpecimenDefinitionTypeTestedContainer
        exclude = ["created_at", "updated_at"]


class SpecimenDefinitionTypeTestedHandlingSerializer(WritableNestedModelSerializer):
    """Serializer for specimen definition type tested handling."""

    temperature_qualifier = CodeableConceptSerializer(many=False, required=False)
    temperature_range = RangeSerializer(many=False, required=False)
    max_duration = QuantitySerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = SpecimenDefinitionTypeTestedHandling
        exclude = ["created_at", "updated_at"]


class SpecimenDefinitionTypeTestedSerializer(WritableNestedModelSerializer):
    """Serializer for specimen definition type tested."""

    type = CodeableConceptSerializer(many=False, required=False)
    container = SpecimenDefinitionTypeTestedContainerSerializer(
        many=False, required=False
    )
    retention_time = QuantitySerializer(many=False, required=False)
    retention_criterion = CodeableConceptSerializer(many=True, required=False)
    handling = SpecimenDefinitionTypeTestedHandlingSerializer(many=True, required=False)
    testing_destination = CodeableConceptSerializer(many=True, required=False)

    class Meta:
        """meta options."""

        model = SpecimenDefinitionTypeTested
        exclude = ["created_at", "updated_at"]


class SpecimenDefinitionSerializer(BaseWritableNestedModelSerializer):
    """Serializer for specimen definition."""

    identifier = IdentifierSerializer(many=False, required=False)
    version_algorithm_coding = CodingSerializer(many=False, required=False)
    subject_codeable_concept = CodeableConceptSerializer(many=False, required=False)
    subject_reference = GroupReferenceSerializer(many=False, required=False)
    contact = ContactDetailSerializer(many=True, required=False)
    use_context = UsageContextSerializer(many=True, required=False)
    jurisdiction = CodeableConceptSerializer(many=True, required=False)
    effective_period = PeriodSerializer(many=False, required=False)
    type_collected = CodeableConceptSerializer(many=False, required=False)
    patient_preparation = CodeableConceptSerializer(many=True, required=False)
    collection = CodeableConceptSerializer(many=True, required=False)
    type_tested = SpecimenDefinitionTypeTestedSerializer(many=True, required=False)

    class Meta:
        """meta options."""

        model = SpecimenDefinition
        exclude = ["created_at", "updated_at"]



"""device serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AnnotationSerializer,
    AttachmentSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    ContactPointSerializer,
    IdentifierSerializer,
    OrganizationReferenceSerializer,
    QuantitySerializer,
    RangeSerializer,
)
from dfhir.devicedefinitions.serializers import DeviceDefinitionReferenceSerializer
from dfhir.endpoints.serializers import EndpointReferenceSerializer
from dfhir.locations.serializers import LocationReferenceSerializer

from .models import (
    Device,
    DeviceCodeableReference,
    DeviceConformsTo,
    DeviceDeviceDefinitionCodeableReference,
    DeviceDeviceDefinitionReference,
    DeviceDeviceMetricReference,
    DeviceName,
    DeviceProperty,
    DeviceReference,
    DeviceUdiCarrier,
    DeviceVersion,
)


class DeviceUdiCarrierSerializer(WritableNestedModelSerializer):
    """Device UDI carrier serializer."""

    class Meta:
        """Meta class."""

        model = DeviceUdiCarrier
        exclude = ["created_at", "updated_at"]


class DeviceNameSerializer(WritableNestedModelSerializer):
    """Device name serializer."""

    type = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = DeviceName
        exclude = ["created_at", "updated_at"]


class DeviceVersionSerializer(WritableNestedModelSerializer):
    """Device version serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    component = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = DeviceVersion
        exclude = ["created_at", "updated_at"]


class DeviceConformsToSerializer(WritableNestedModelSerializer):
    """Device conforms to serializer."""

    category = CodeableConceptSerializer(many=False, required=False)
    specification = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = DeviceConformsTo
        exclude = ["created_at", "updated_at"]


class DevicePropertySerializer(WritableNestedModelSerializer):
    """Device property serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    value_range = RangeSerializer(many=False, required=False)
    value_quantity = QuantitySerializer(many=False, required=False)
    value_codeable_concept = CodeableConceptSerializer(many=False, required=False)
    value_attachment = AttachmentSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = DeviceProperty
        exclude = ["created_at", "updated_at"]


class DeviceReferenceSerializer(WritableNestedModelSerializer):
    """Device reference serializer."""

    identifier = IdentifierSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = DeviceReference
        exclude = ["created_at", "updated_at"]


class DeviceCodeableReferenceSerializer(WritableNestedModelSerializer):
    """Device codeable reference serializer."""

    concept = CodeableConceptSerializer(many=False, required=False)
    reference = DeviceReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = DeviceCodeableReference
        exclude = ["created_at", "updated_at"]


class DeviceSerializer(BaseWritableNestedModelSerializer):
    """Device serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    definition = DeviceDefinitionReferenceSerializer(many=False, required=False)
    udi_carrier = DeviceUdiCarrierSerializer(many=True, required=False)
    availability_status = CodeableConceptSerializer(many=False, required=False)
    biological_source_event = IdentifierSerializer(many=False, required=False)
    name = DeviceNameSerializer(many=True, required=False)
    category = CodeableConceptSerializer(many=True, required=False)
    type = CodeableConceptSerializer(many=True, required=False)
    device_version = DeviceVersionSerializer(many=True, required=False)
    conforms_to = DeviceConformsToSerializer(many=True, required=False)
    property = DevicePropertySerializer(many=True, required=False)
    mode = CodeableConceptSerializer(many=False, required=False)
    duration = QuantitySerializer(many=False, required=False)
    owner = OrganizationReferenceSerializer(many=False, required=False)
    contact = ContactPointSerializer(many=True, required=False)
    location = LocationReferenceSerializer(many=False, required=False)
    endpoint = EndpointReferenceSerializer(many=False, required=False)
    gateway = DeviceCodeableReferenceSerializer(many=True, required=False)
    note = AnnotationSerializer(many=True, required=False)
    safety = CodeableConceptSerializer(many=True, required=False)
    parent = DeviceReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = Device
        exclude = ["created_at", "updated_at"]


class DeviceDeviceMetricReferenceSerializer(BaseReferenceModelSerializer):
    """Device device metric reference serializer."""

    identifier = IdentifierSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = DeviceDeviceMetricReference
        exclude = ["created_at", "updated_at"]


class DeviceDeviceDefinitionReferenceSerializer(BaseReferenceModelSerializer):
    """Device device definition reference serializer."""

    identifier = IdentifierSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = DeviceDeviceDefinitionReference
        exclude = ["created_at", "updated_at"]


class DeviceDeviceDefinitionCodeableReferenceSerializer(WritableNestedModelSerializer):
    """Device device definition codeable reference serializer."""

    reference = DeviceDeviceDefinitionReferenceSerializer(many=False, required=False)
    concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = DeviceDeviceDefinitionCodeableReference
        exclude = ["created_at", "updated_at"]



"""Slot choices for the slots app."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class AppointmentType(models.TextChoices):
    """Appointment type choices."""

    WALK_IN = "walk-in", _("Walk-in")
    ROUTINE = "routine", _("Routine")
    CHECKUP = "checkup", _("Check-up")
    FOLLOWUP = "followup", _("Follow-up")
    EMERGENCY = "emergency", _("Emergency")


class SlotStatus(models.TextChoices):
    """Slot status choices."""

    FREE = "free", _("Free")
    BUSY = "busy", _("Busy")
    BUSY_UNAVAILABLE = "busy-unavailable", _("Busy Unavailable")
    BUSY_TENTATIVE = "busy-tentative", _("Busy Tentative")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")



"""molecular definition serializers."""

from drf_writable_nested import WritableNestedModelSerializer
from rest_framework import serializers

from dfhir.base.serializers import (
    BaseReferenceModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    QuantitySerializer,
    RangeSerializer,
)
from dfhir.documentreferences.serializers import DocumentReferenceReferenceSerializer
from dfhir.moleculardefinitions.models import (
    MolecularDefinition,
    MolecularDefinitionCoordinateInterval,
    MolecularDefinitionCoordinateIntervalCoordinateSystem,
    MolecularDefinitionCytobandInterval,
    MolecularDefinitionCytobandIntervalStartEndCytoband,
    MolecularDefinitionCytobandLocation,
    MolecularDefinitionLocation,
    MolecularDefinitionReference,
    MolecularDefinitionRepresentation,
    MolecularDefinitionRepresentationConcatenated,
    MolecularDefinitionRepresentationConcatenatedSequenceElement,
    MolecularDefinitionRepresentationExtracted,
    MolecularDefinitionRepresentationLiteral,
    MolecularDefinitionRepresentationRelative,
    MolecularDefinitionRepresentationRelativeEdit,
    MolecularDefinitionRepresentationRepeated,
    MolecularDefinitionSequenceLocation,
    MolecularSequenceGenomeAssembly,
)


class MolecularDefinitionReferenceSerializer(BaseReferenceModelSerializer):
    """molecular definition reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = MolecularDefinitionReference
        exclude = ["created_at", "updated_at"]


class MolecularDefinitionCoordinateIntervalCoordinateSystemSerializer(
    WritableNestedModelSerializer
):
    """molecular definition coordinate interval coordinate system serializer."""

    system = CodeableConceptSerializer(required=False)
    origin = CodeableConceptSerializer(required=False)
    normalization_method = CodeableConceptSerializer(required=False)
    start_quantity = QuantitySerializer(required=False)
    start_range = RangeSerializer(required=False)
    end_quantity = QuantitySerializer(required=False)
    end_range = RangeSerializer(required=False)

    class Meta:
        """Meta options."""

        model = MolecularDefinitionCoordinateIntervalCoordinateSystem
        exclude = ["created_at", "updated_at"]


class MolecularDefinitionCoordinateIntervalSerializer(WritableNestedModelSerializer):
    """molecular definition coordinate interval serializer."""

    coordinate_system = MolecularDefinitionCoordinateIntervalCoordinateSystemSerializer(
        required=False
    )

    class Meta:
        """Meta options."""

        model = MolecularDefinitionCoordinateInterval
        exclude = ["created_at", "updated_at"]


class MolecularDefinitionRepresentationLiteralSerializer(WritableNestedModelSerializer):
    """molecular definition representation literal serializer."""

    encoding = CodeableConceptSerializer(required=False)

    class Meta:
        """meta options."""

        model = MolecularDefinitionRepresentationLiteral
        exclude = ["created_at", "updated_at"]


class MolecularDefinitionRepresentationExtractedSerializer(
    WritableNestedModelSerializer
):
    """molecular definition representation extracted serializer."""

    string_molecule = MolecularDefinitionReferenceSerializer(required=False)
    coordinate_system = CodeableConceptSerializer(required=False)

    class Meta:
        """meta options."""

        model = MolecularDefinitionRepresentationExtracted
        exclude = ["created_at", "updated_at"]


class MolecularDefinitionRepresentationRepeatedSerializer(
    WritableNestedModelSerializer
):
    """molecular definition representation repeated serializer."""

    sequence_motif = MolecularDefinitionReferenceSerializer(required=False)

    class Meta:
        """meta options."""

        model = MolecularDefinitionRepresentationRepeated
        exclude = ["created_at", "updated_at"]


class MolecularDefinitionRepresentationConcatenatedSequenceElementSerializer(
    WritableNestedModelSerializer
):
    """molecular definition representation concatenated sequence element serializer."""

    sequence = MolecularDefinitionReferenceSerializer(required=False)

    class Meta:
        """meta options."""

        model = MolecularDefinitionRepresentationConcatenatedSequenceElement
        exclude = ["created_at", "updated_at"]


class MolecularDefinitionRepresentationConcatenatedSerializer(
    WritableNestedModelSerializer
):
    """molecular definition representation concatenated sequence element serializer."""

    sequence_element = (
        MolecularDefinitionRepresentationConcatenatedSequenceElementSerializer(
            required=False, many=True
        )
    )

    class Meta:
        """meta options."""

        model = MolecularDefinitionRepresentationConcatenated
        exclude = ["created_at", "updated_at"]


class MolecularDefinitionRepresentationRelativeEditSerializer(
    WritableNestedModelSerializer
):
    """molecular definition representation relative edit serializer."""

    coordinate_system = CodeableConceptSerializer(required=False)
    replacement_molecule = MolecularDefinitionReferenceSerializer(required=False)
    replaced_molecule = CodeableConceptSerializer(required=False)

    class Meta:
        """meta options."""

        model = MolecularDefinitionRepresentationRelativeEdit
        exclude = ["created_at", "updated_at"]


class MolecularDefinitionRepresentationRelativeSerializer(
    WritableNestedModelSerializer
):
    """molecular definition representation relative serializer."""

    starting_molecule = MolecularDefinitionReferenceSerializer(required=False)
    edit = MolecularDefinitionRepresentationRelativeEditSerializer(
        required=False, many=True
    )

    class Meta:
        """meta options."""

        model = MolecularDefinitionRepresentationRelative
        exclude = ["created_at", "updated_at"]


class MolecularDefinitionRepresentationSerializer(WritableNestedModelSerializer):
    """molecular definition representation serializer."""

    focus = CodeableConceptSerializer(required=False)
    code = CodeableConceptSerializer(required=False)
    literal = MolecularDefinitionRepresentationLiteralSerializer(required=False)
    resolvable = DocumentReferenceReferenceSerializer(required=False)
    extracted = MolecularDefinitionRepresentationExtractedSerializer(required=False)
    repeated = MolecularDefinitionRepresentationRepeatedSerializer(required=False)
    concatenated = MolecularDefinitionRepresentationConcatenatedSerializer(
        required=False
    )
    relative = MolecularDefinitionRepresentationRelativeSerializer(required=False)

    class Meta:
        """meta options."""

        model = MolecularDefinitionRepresentation
        exclude = ["created_at", "updated_at"]


class MolecularDefinitionSequenceLocationSerializer(WritableNestedModelSerializer):
    """molecular definition sequence location serializer."""

    sequence_context = MolecularDefinitionReferenceSerializer(required=False)
    coordinated_interval = MolecularDefinitionCoordinateIntervalSerializer(
        required=False
    )
    strand = CodeableConceptSerializer(required=False)

    class Meta:
        """meta options."""

        model = MolecularDefinitionSequenceLocation
        exclude = ["created_at", "updated_at"]


class MolecularSequenceGenomeAssemblySerializer(WritableNestedModelSerializer):
    """molecular sequence genome assembly serializer."""

    organism = CodeableConceptSerializer(required=False)
    build = CodeableConceptSerializer(required=False)
    accession = CodeableConceptSerializer(required=False)

    class Meta:
        """meta options."""

        model = MolecularSequenceGenomeAssembly
        exclude = ["created_at", "updated_at"]


class MolecularDefinitionCytobandIntervalStartEndCytobandSerializer(
    serializers.ModelSerializer
):
    """molecular definition cytoband interval start/end serializer."""

    class Meta:
        """meta options."""

        model = MolecularDefinitionCytobandIntervalStartEndCytoband
        exclude = ["created_at", "updated_at"]


class MolecularDefinitionCytobandIntervalSerializer(WritableNestedModelSerializer):
    """molecular definition cytoband interval serializer."""

    chromosome = CodeableConceptSerializer(required=False)
    start_cytoband = MolecularDefinitionCytobandIntervalStartEndCytobandSerializer(
        required=False
    )
    end_cytoband = MolecularDefinitionCytobandIntervalStartEndCytobandSerializer(
        required=False
    )

    class Meta:
        """meta options."""

        model = MolecularDefinitionCytobandInterval
        exclude = ["created_at", "updated_at"]


class MolecularDefinitionCytobandLocationSerializer(WritableNestedModelSerializer):
    """molecular definition cytoband location serializer."""

    genome_assembly = MolecularSequenceGenomeAssemblySerializer(required=False)
    cytoband_interval = MolecularDefinitionCytobandIntervalSerializer(required=False)

    class Meta:
        """meta options."""

        model = MolecularDefinitionCytobandLocation
        exclude = ["created_at", "updated_at"]


class MolecularDefinitionLocationSerializer(WritableNestedModelSerializer):
    """molecular definition location serializer."""

    sequence_location = MolecularDefinitionSequenceLocationSerializer(required=False)
    cytoband_location = MolecularDefinitionCytobandLocationSerializer(required=False)

    class Meta:
        """meta options."""

        model = MolecularDefinitionLocation
        exclude = ["created_at", "updated_at"]


class MolecularDefinitionSerializer(WritableNestedModelSerializer):
    """Serializer for MolecularDefinition model."""

    identifier = IdentifierSerializer(required=False, many=True)
    molecule_type = CodeableConceptSerializer(required=False)
    type = CodeableConceptSerializer(required=False, many=True)
    topology = CodeableConceptSerializer(required=False, many=True)
    location = MolecularDefinitionLocationSerializer(many=True, required=False)
    member_state = MolecularDefinitionReferenceSerializer(required=False, many=True)
    representation = MolecularDefinitionRepresentationSerializer(
        required=False, many=True
    )

    class Meta:
        """meta options."""

        model = MolecularDefinition
        exclude = ["created_at", "updated_at"]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0004_initial'),
        ('careplans', '0004_initial'),
        ('nutritionintakes', '0001_initial'),
        ('patients', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='nutritionintake',
            name='subject',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_subject', to='patients.patientgroupreference'),
        ),
        migrations.AddField(
            model_name='nutritionintakebasedonreference',
            name='care_plan',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_based_on_reference_care_plan', to='careplans.careplan'),
        ),
        migrations.AddField(
            model_name='nutritionintakebasedonreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_based_on_reference_identifier', to='base.identifier'),
        ),
    ]



"""Advere Event choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class AdverseEventStatusChoices(models.TextChoices):
    """Adverse Event status choices."""

    IN_PROGRESS = "in-progress", _("in-progress")
    COMPLETED = "completed", _("completed")
    ENTERED_IN_ERROR = "entered-in-error", _("entered-in-error")
    UNKNOWN = "unknown", _("unknown")


class AdverseEventActualityChoices(models.TextChoices):
    """Adverse Event actuality choices."""

    ACTUAL = "actual", _("actual")
    POTENTIAL = "potential", _("potential")



"""Endpoints views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import Endpoint
from .serializers import EndpointSerializer


class EndpointListView(APIView):
    """Endpoint List Views."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200, EndpointSerializer(many=True)})
    def get(self, request):
        """Get all endpoints."""
        endpoint = Endpoint.objects.all()
        serializer = EndpointSerializer(endpoint, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(request=EndpointSerializer, responses={201, EndpointSerializer})
    def post(self, request):
        """Create and endpoint."""
        serializer = EndpointSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        endpoint = serializer.save()
        detailed_serializer = EndpointSerializer(endpoint)
        return Response(detailed_serializer.data, status=status.HTTP_201_CREATED)


class EndpointDetailView(APIView):
    """Endpoint Serializer Detail View."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get endpoint object."""
        try:
            return Endpoint.objects.get(pk=pk)
        except Endpoint.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200, EndpointSerializer})
    def get(self, request, pk):
        """Get an endpoint."""
        endpoint = self.get_object(pk)
        serializer = EndpointSerializer(endpoint)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(request=EndpointSerializer, responses={200, EndpointSerializer})
    def put(self, request, pk):
        """Update an endpoint."""
        endpoint = self.get_object(pk)
        serializer = EndpointSerializer(endpoint, data=request.data)
        serializer.is_valid(raise_exception=True)
        endpoint = serializer.save()
        detailed_serializer = EndpointSerializer(endpoint)
        return Response(detailed_serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204})
    def delete(self, request, pk):
        """Delete an endpoint."""
        endpoint = self.get_object(pk)
        endpoint.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""medication statements models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    Reference,
    TimeStampedModel,
)
from dfhir.medicationstatements.choices import MedicationStatementStatusChoices


class MedicationStatementPartOfReference(BaseReference):
    """medication statement part of reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_statement_part_of_reference_identifier",
    )
    procedure = models.ForeignKey(
        "procedures.Procedure",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_statement_part_of_reference_procedure",
    )
    medication_statement = models.ForeignKey(
        "MedicationStatement",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_statement_part_of_reference_medication_statement",
    )


class MedicationStatementInformationSourceReference(BaseReference):
    """medication statement information source reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_statement_information_source_reference_identifier",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_statement_information_source_reference_device",
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        related_name="medication_statement_information_source_reference_patient",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        related_name="medication_statement_information_source_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        related_name="medication_statement_information_source_reference_practitioner_role",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        related_name="medication_statement_information_source_reference_related_person",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        related_name="medication_statement_information_source_reference_organization",
        null=True,
    )


class MedicationStatementReasonReference(BaseReference):
    """medication statement reason reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_statement_reason_reference_identifier",
    )
    condition = models.ForeignKey(
        "conditions.Condition",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_statement_reason_reference_condition",
    )
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_statement_reason_reference_observation",
    )
    diagnostic_report = models.ForeignKey(
        "diagnosticreports.DiagnosticReport",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_statement_reason_reference_diagnostic_report",
    )
    procedure = models.ForeignKey(
        "procedures.Procedure",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_statement_reason_reference_procedure",
    )


class MedicationStatementAdherence(TimeStampedModel):
    """medication statement adherence model."""

    code = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_statement_adherence_code",
    )
    reason = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_statement_adherence_reason",
    )


class MedicationStatement(TimeStampedModel):
    """medication statement model."""

    identifier = models.ManyToManyField(
        Identifier, blank=True, related_name="medication_statement_identifier"
    )
    part_of = models.ManyToManyField(
        MedicationStatementPartOfReference,
        blank=True,
        related_name="medication_statement_part_of",
    )
    status = models.CharField(
        max_length=255, null=True, choices=MedicationStatementStatusChoices.choices
    )
    category = models.ManyToManyField(
        CodeableConcept, blank=True, related_name="medication_statement_category"
    )
    medication = models.ForeignKey(
        "medications.MedicationCodeableReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_statement_medication",
    )
    subject = models.ForeignKey(
        "patients.PatientGroupReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_statement_subject",
    )
    encounter = models.ForeignKey(
        "encounters.EncounterReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_statement_encounter",
    )
    effective_date_time = models.DateTimeField(null=True)
    effective_period = models.ForeignKey(
        "base.Period",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_statement_effective_period",
    )
    effective_timing = models.ForeignKey(
        "base.Timing",
        on_delete=models.ForeignKey,
        null=True,
        related_name="medication_statement_effective_timing",
    )
    date_asserted = models.DateTimeField(null=True)
    information_source = models.ManyToManyField(
        MedicationStatementInformationSourceReference,
        blank=True,
        related_name="medication_statement_information_source",
    )
    derived_from = models.ManyToManyField(
        Reference, blank=True, related_name="medication_statement_derived_from"
    )
    reason = models.ManyToManyField(
        MedicationStatementReasonReference,
        blank=True,
        related_name="medication_statement_reason",
    )
    note = models.ManyToManyField(
        "base.Annotation", blank=True, related_name="medication_statement_note"
    )
    related_clinical_information = models.TextField(null=True)
    # TODO: dosage = models.ManyToManyField(
    #     "Dosage", blank=True, related_name="medication_statement_dosage"
    # )
    adherence = models.ForeignKey(
        MedicationStatementAdherence,
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="medication_statement_adherence",
    )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('communications', '0002_initial'),
        ('diagnosticreports', '0001_initial'),
        ('encounters', '0002_initial'),
        ('observations', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='conclusioncodereference',
            name='observation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='conclusion_code_observation', to='observations.observation'),
        ),
        migrations.AddField(
            model_name='conclusioncodecodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='conclusion_code_codeable_reference_codeable', to='diagnosticreports.conclusioncodereference'),
        ),
        migrations.AddField(
            model_name='diagnosticreport',
            name='category',
            field=models.ManyToManyField(blank=True, related_name='diagnostics_report_category', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='diagnosticreport',
            name='code',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='diagnostics_report_code', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='diagnosticreport',
            name='communication',
            field=models.ManyToManyField(blank=True, related_name='diagnostic_report_communication', to='communications.communicationreference'),
        ),
        migrations.AddField(
            model_name='diagnosticreport',
            name='conclusion_code',
            field=models.ManyToManyField(blank=True, related_name='diagnostic_report_conclusion_code', to='diagnosticreports.conclusioncodecodeablereference'),
        ),
        migrations.AddField(
            model_name='diagnosticreport',
            name='effective_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='diagnostic_report_effective_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='diagnosticreport',
            name='encounter',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='diagnostic_encounter', to='encounters.encounterreference'),
        ),
        migrations.AddField(
            model_name='diagnosticreport',
            name='identifier',
            field=models.ManyToManyField(blank=True, related_name='diagnostic_report_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='diagnosticreport',
            name='note',
            field=models.ManyToManyField(blank=True, related_name='diagnostic_report_note', to='base.annotation'),
        ),
        migrations.AddField(
            model_name='diagnosticreport',
            name='presented_form',
            field=models.ManyToManyField(blank=True, to='base.attachment'),
        ),
        migrations.AddField(
            model_name='diagnosticreport',
            name='recommendation',
            field=models.ManyToManyField(blank=True, to='base.codeablereference'),
        ),
        migrations.AddField(
            model_name='diagnosticreport',
            name='result',
            field=models.ManyToManyField(blank=True, related_name='diagnostic_result', to='observations.observationreference'),
        ),
    ]



"""Device metrics models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    Quantity,
    TimeStampedModel,
)

from . import choices


class DeviceMetricCalibration(TimeStampedModel):
    """Device Metric Calibration model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="device_metric_calibration_type",
        null=True,
    )
    state = models.CharField(
        max_length=255, choices=choices.DeviceMetricCalibrationState.choices, null=True
    )
    time = models.DateTimeField(null=True)


class DeviceMetric(TimeStampedModel):
    """Device Metric model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="device_metrics_identifier", blank=True
    )
    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="device_metric_type",
        null=True,
    )
    unit = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="device_metric_unit",
        null=True,
    )
    device = models.ForeignKey(
        "devices.DeviceReference",
        on_delete=models.DO_NOTHING,
        related_name="device_metric_device",
        null=True,
    )
    operational_status = models.CharField(
        max_length=255, choices=choices.DeviceMetricOperationalStatus.choices, null=True
    )
    color = models.CharField(max_length=255, null=True)
    category = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="device_metric_category",
    )
    measurement_frequency = models.ForeignKey(
        Quantity,
        on_delete=models.DO_NOTHING,
        related_name="device_metric_measurement_frequency",
        null=True,
    )
    calibration = models.ManyToManyField(
        DeviceMetricCalibration, related_name="device_metric_calibration", blank=True
    )


class DeviceMetricReference(BaseReference):
    """Device Metric Reference model."""

    device_metric = models.ForeignKey(
        DeviceMetric,
        on_delete=models.DO_NOTHING,
        related_name="device_metric_reference",
        null=True,
    )
    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="device_metric_reference_identifier",
        null=True,
    )



"""Detected issues serializers."""

from drf_writable_nested import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    PeriodSerializer,
    ReferenceSerializer,
)
from dfhir.encounters.serializers import EncounterReferenceSerializer
from dfhir.practitioners.serializers import (
    PractitionerPractitionerRoleReferenceSerializer,
)

from .models import (
    DetectedIssue,
    DetectedIssueAuthorReference,
    DetectedIssueCodeableReference,
    DetectedIssueEvidence,
    DetectedIssueMitigation,
    DetectedIssueReference,
    DetectedIssueSubjectReference,
)


class DetectedIssueReferenceSerializer(BaseReferenceModelSerializer):
    """detected Issue reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = DetectedIssueReference
        exclude = ["created_at", "updated_at"]


class DetectedIssueCodeableReferenceSerializer(WritableNestedModelSerializer):
    """detected issue codeable reference serializer."""

    concept = CodeableConceptSerializer(required=False)
    reference = DetectedIssueReferenceSerializer(required=False)

    class Meta:
        """meta options."""

        model = DetectedIssueCodeableReference
        exclude = ["created_at", "updated_at"]


class DetectedIssueSubjectReferenceSerializer(BaseReferenceModelSerializer):
    """Detected Issue Subject Reference Serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """Meta."""

        model = DetectedIssueSubjectReference
        exclude = ["created_at", "updated_at"]


class DetectedIssueAuthorReferenceSerializer(BaseReferenceModelSerializer):
    """Detected Issue Author Reference Serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """Meta."""

        model = DetectedIssueAuthorReference
        exclude = ["created_at", "updated_at"]


class DetectedIssueEvidenceSerializer(BaseWritableNestedModelSerializer):
    """Detected Issue Evidence Serializer."""

    code = CodeableConceptSerializer(required=True, many=False)
    detail = ReferenceSerializer(required=False, many=True)

    class Meta:
        """Meta."""

        model = DetectedIssueEvidence
        exclude = ["created_at", "updated_at"]


class DetectedIssueMitigationSerializer(BaseWritableNestedModelSerializer):
    """Detected Issue Mitigation Serializer."""

    action = CodeableConceptSerializer(required=False, many=False)
    author = PractitionerPractitionerRoleReferenceSerializer(required=False, many=False)
    note = AnnotationSerializer(required=False, many=True)

    class Meta:
        """Meta."""

        model = DetectedIssueMitigation
        exclude = ["created_at", "updated_at"]


class DetectedIssueSerializer(BaseWritableNestedModelSerializer):
    """Detected Issue Serializer."""

    identifier = IdentifierSerializer(required=False, many=True)
    category = CodeableConceptSerializer(required=False, many=True)
    code = CodeableConceptSerializer(required=False, many=False)
    subject = DetectedIssueSubjectReferenceSerializer(required=False, many=False)
    encounter = EncounterReferenceSerializer(required=False, many=False)
    identified_period = PeriodSerializer(required=False, many=False)
    author = DetectedIssueAuthorReferenceSerializer(required=False, many=False)
    implicated = ReferenceSerializer(required=False, many=True)
    evidence = DetectedIssueEvidenceSerializer(required=False, many=True)
    mitigation = DetectedIssueMitigationSerializer(required=False, many=True)

    class Meta:
        """Meta."""

        model = DetectedIssue
        exclude = ["created_at", "updated_at"]



"""Slots views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .filters import SlotsFilter
from .models import Slot
from .serializers import SlotSerializer


class SlotListView(APIView):
    """Slot list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: SlotSerializer(many=True)})
    def get(self, request):
        """Get slots."""
        slots = Slot.objects.all()
        filterd_slots = SlotsFilter(request.GET, queryset=slots)
        serializer = SlotSerializer(filterd_slots.qs, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    def post(self, request):
        """Create a new slot."""
        return Response(status=status.HTTP_405_METHOD_NOT_ALLOWED)


class SlotDetailView(APIView):
    """Slot detail view."""

    def get_object(self, pk):
        """Get slot object."""
        try:
            return Slot.objects.get(pk=pk)
        except Slot.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: SlotSerializer})
    def get(self, request, pk=None):
        """Get slot."""
        slot = self.get_object(pk)
        serializer = SlotSerializer(slot)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={200: SlotSerializer})
    def patch(self, request, pk=None):
        """Update slot."""
        slot = self.get_object(pk)
        serializer = SlotSerializer(slot, data=request.data, partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)
